!> \~japanese
!! \brief 地球における物理定数をまとめたモジュール.
!! \~english
!! \brief This module collects the physical constants.
!! \~
!!
!! <br><br>
!! \b Copyright (C) GFD Dennou Club, 2011-2012. All rights reserved. <br>
!! \b license ?? <br>
!! \author Yuta Kawai
!!
module Earth_Constants

  ! モジュール引用 ; Use statements
  !
  use dc_types, only : DP

  ! 宣言文 ; Declaration statements
  !
  implicit none
  private

  !> \~japanese 地球の平均半径 [m].
  !! \~english The average radius of the earth [m].
  real(DP), parameter, public :: earth_radius = 6.378137e06 

  !> \~japanese 地球の重力加速度 [m \f$\rm s^{-2}\f$].
  !! \~english Gravitational acceleration [m \f$\rm s^{-2}\f$].
  real(DP), parameter, public :: earth_g      = 9.80665e00

  !> \~japanese 地球の自転角速度 [ rad \f$ \rm s^{-1}\f$ ].
  !! \~english The angular speed of Earth's rotation [ rad \f$ \rm s^{-1}\f$ ].
  real(DP), parameter, public :: earth_Omega  = 7.2921159e-05 

end module Earth_Constants
