!= IGMBaseLib の core, util パッケージのテストプログラムのメインプログラム
! 正二十面体格子の生成および微分演算を提供するクラスをテストする.  
! 微分演算のテストでは, 数値解と解析解から誤差ノルムを計算することで, 数値誤差を定量的に見積もる.  
!
! Authors::   Yuta Kawai
! Version::   $Id: diffOptr_test.f90,v 1.9 2011-07-01 09:11:03 ykawai Exp $
! Tag Name::  $Name:  $
! Copyright:: Copyright (C) GFD Dennou Club, 2011-2012. All rights reserved.
! License::   See COPYRIGHT[link:../../COPYRIGHT]
!
program main

  ! モジュール引用 ; Use statements
  !

  !
  !
  use dc_types, only: DP, &
    &                 TOKEN, &
    &                 STRING
  
  !
  !
  !
  use IcGrid2D_FVM_Manager
  
  !
  !
  !
  use Field_IcGrid2D_Manager

  !
  !
  !
  use IcGrid_ncWriter_mod
  
  !
  !
  !
  use IcGrid2D_FVM_Builder, only: &
    & construct_icosahedral_grid, &
    & GTYPE_STDGRID, GTYPE_STDGCGRID, GTYPE_SPRGCGRID

  !
  !
  !
  use IcGrid_ncGridLoader, only: load_gridData

  !
  !
  !
  use Derivate_Field_IcGrid2D_manager, only: &
    & Derivate_Field_IcGrid2D, &
    & Derivate_Field_IcGrid2D_Init, Derivate_Field_IcGrid2D_Final, &
    & divergence_op, gradient_op, vertical_curl_op

  !
  !
  !
  use Field_IcGrid2D_error_norm_check
  
  !
  !
  !
  use test_diff_field, only: field, dfield, ana_dfield,  &
    &  init_test_diff_field, create_field, calc_ana_dfield, &
    &  DIV_OPERATOR, GRADIENT_OPERATOR, ROT_OPERATOR 
  
  ! 宣言文 ; Declaration statements
  !

  implicit none
  
  type(IcGrid2D_FVM) :: ic_grid

  !!!!
  !

!  character(TOKEN) :: load_gridtype_name = GRIDTYPE_SPRGCGRID_NAME
  integer, parameter :: grid_type_id = GTYPE_SPRGCGRID

  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  character(STRING) :: grid_filename = './SPR-GC-grid_glevel5.nc'
  logical :: load_flag = .true.
  
  integer, parameter :: glevel = 5
  real(DP), parameter :: ic_radius = 1.0d0
  integer :: m = 3
  integer :: n = 3

  type(Derivate_Field_IcGrid2D) :: diff_eval

  integer :: rcID, i, j
  integer :: EMin, EMax, idMin, idMax, EffSize
 
  ! 格子生成
  call IcGrid2D_FVM_Init(ic_grid, glevel, ic_radius )

  if ( load_flag ) then
    call load_gridData( grid_filename, ic_grid )
  else
    call construct_grid( grid_filename, grid_type_id )
  end if

  ! 微分演算クラスの初期化
  call Derivate_Field_IcGrid2D_Init(diff_eval, ic_grid)

  ! 場を保存する配列を確保
  EMin = get_EffSize_Min(ic_grid)
  EMax = get_EffSize_Max(ic_grid)
  idMin = get_IdMin(ic_grid)
  idMax = get_IdMax(ic_grid)
  EffSize = EMax - EMin + 1
  
  call init_test_diff_field(ic_grid)
  
  !
  ! 各微分演算子の数値誤差の検証
  !
  call test_differential_operator(DIV_OPERATOR, m, n)
  call test_differential_operator(ROT_OPERATOR, m, n)
  call test_differential_operator(GRADIENT_OPERATOR, m, n)

  call Derivate_Field_IcGrid2D_Final(diff_eval)

contains
subroutine construct_grid(file_name, grid_type_id)
  !
  !
  !

  !
  !
  use IcGrid_ncWriter_mod, only: IcGrid_ncWriter, &
    & IcGrid_ncWriter_Init, open_ncFile, ncdef_GridData, end_ncdef, &
    & write_GridData, close_ncFile

  !
  !
  character(*), intent(in) :: file_name
  integer, intent(in) :: grid_type_id

  !
  !
  type(IcGrid_ncWriter) :: writer
 
  !
  call construct_icosahedral_grid(grid_type_id, ic_grid)

  !
  call IcGrid_ncWriter_Init(writer, ic_grid)
  call open_ncFile(writer, file_name)
  call ncdef_GridData(writer)
  call end_ncdef(writer)
  call write_GridData(writer)
  call close_ncFile(writer)

end subroutine construct_grid

subroutine test_differential_operator(diffOptrType, m, n)
  !
  !
  !
  integer, intent(in) :: diffOptrType 
  integer, intent(in) :: m, n
   
  !
  !
  character(len=10) :: str_diffOptrType
  integer :: dfield_dim
  !
  !

  ! ベクトル場の作成
  call create_field( ic_grid, diffOptrType, m, n )

  ! 場の微分を数値的に評価する
  !

  !
  call paste_field2D_margin(field)
  ! 
  select case(diffOptrType)
    case(DIV_OPERATOR)
      dfield_dim = 1
      call divergence_op(diff_eval, field, dfield)
    case(ROT_OPERATOR)
      dfield_dim = 1
      call vertical_curl_op(diff_eval, field, dfield)
    case(GRADIENT_OPERATOR)
      dfield_dim = 3
      call gradient_op(diff_eval, field, dfield)
  end select


  ! 場の微分を解析的に評価する
  call calc_ana_dfield( ic_grid, diffOptrType, dfield_dim, m, n )

  ! 数値的に求めた場と解析に求めた場のデータから, 誤差ノルムを計算する. 
  write(str_diffOptrType, '(i1)') diffOptrType
  call check_norm(str_diffOptrType , dfield_dim)

end subroutine test_differential_operator

subroutine check_norm(str_optr_type, dfield_dim)

  ! モジュール引用 ; Use statements
  !

  !
  !
  use rcregions_norm_check

  !
  !
  character(*), intent(in) :: str_optr_type
  integer, intent(in) :: dfield_dim

  !
  !
  !
  real(DP) :: norm_1, norm_2, norm_3

  !
  !

  write(*,*) '===== Numerical error test: ', str_optr_type

  norm_1 = calc_norm1( ana_dfield%val(:,EMin:EMax,EMin:EMax,1:dfield_dim), &
    &                  dfield%val(:,EMin:EMax,EMin:EMax,1:dfield_dim))
  write(*,*) 'norm1:', norm_1

  norm_2 = calc_norm2( ana_dfield%val(:,EMin:EMax,EMin:EMax,1:dfield_dim), &
    &                  dfield%val(:,EMin:EMax,EMin:EMax,1:dfield_dim) )

  write(*,*) 'norm2:', norm_2

  norm_3 = calc_norm3( ana_dfield%val(:,EMin:EMax,EMin:EMax,1:dfield_dim), &
       &               dfield%val(:,EMin:EMax,EMin:EMax,1:dfield_dim), .true. )
  write(*,*) 'norm3:', norm_3

end subroutine check_norm
end program
 
